@@ -7,20 +7,17 @@
 #' @param x A strel output object (list)
 #' @param estimate A character string indicating what estimate to plot from the strel output object
 #' @param low.bound A number for the threshold to be tested against
-#'
 #' @examples
-#' p_strel(strel(asrm, "lambda2"), "lambda2", .80)
+#' p_strel(strel(asrm, "lambda2", n.chains = 2, n.iter = 100, freq = FALSE), "lambda2", .80)
 #' @export
-
-
 p_strel <- function(x, estimate, low.bound){
   posi1 <- grep(estimate, x$estimates, ignore.case = T)
-  samp <- unlist(x$Bayes$samp[posi1])
+  samp <- as.vector(x$Bayes$samp[[posi1]])
   obj <- ecdf(samp)
   post_prob <- 1 - obj(low.bound)
 
   # prior prob
-  n.item <- dim(x$Bayes$covsamp)[2]
+  n.item <- dim(x$Bayes$covsamp)[3]
   prior_all <- priors[[as.character(n.item)]]
   posi2 <- grep(estimate, prior_all, ignore.case = T)
   prior <- prior_all[[posi2]]
